#ifndef DATAPARSEROUTPUT_H
#define DATAPARSEROUTPUT_H

#include <QObject>
#include <Data/FileData.h>
#include <Data/Datas.h>
#include <Core/tools.h>
#include <QFile>
#include <QXmlStreamWriter>
#include <QDomDocument>


class DataParserOutput : public QObject
{
    Q_OBJECT
public:
    explicit DataParserOutput(QObject *parent = 0);
    void createFileDataList(const QList<FileData>& fileDataList,const QString& filename);
    void createAuthMessage(ClientAutorization *auth);
    void createServerSettings(ServerSettings *serverSettings);
    void changeVersion(StreamingVersionData *versionData);
    void createAuthData(ServerAuthorization *serverAuth);
    void createAuthDataOffline(const QString& username,const QString& pass);
    void addRunData(QList<int> displays);
    void saveClientSettings(const QString& language, bool isAutoStart);
    QList<FileData> *xmlFileDataParse(const QByteArray& array,const QString& filter = "");
    QByteArray xmlAnswer_notify(const QString& code);
    QByteArray xmlAnswer(const QList<SXmlAnswerTag>& listTag, const QString& elemUp1 = "", const QString& elemUp2 = "");
private:
    const QString XMLLanguageProperty = "Language=\"";
    const QString XMLAutoStartProperty = "AutoStart=\"";
    ClientAutorization *authPassCache;
};

#endif // DATAPARSEROUTPUT_H
